@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: base64tool.cpp 2745 2005-01-30 18:28:07Z mosu $
+   $Id: base64tool.cpp 3335 2006-11-03 15:51:09Z mosu $
 
    command line parameter parsing, looping, output handling
 
@@ -26,16 +26,16 @@
 using namespace std;
 
 void
-usage(int retval) {
-  mxinfo(_(
+set_usage() {
+  usage_text = _(
     "base64util <encode|decode> <input> <output> [maxlen]\n"
     "\n"
     "  encode - Read from <input>, encode to Base64 and write to <output>.\n"
     "           Max line length can be specified and is 72 if left out.\n"
     "  decode - Read from <input>, decode to binary and write to <output>.\n"
-    ));
+    );
 
-  exit(retval);
+  version_info = "base64util v" VERSION;
 }
 
 int
@@ -49,6 +49,9 @@ main(int argc,
   mm_io_c *in, *out;
   mm_text_io_c *intext;
 
+  init_stdio();
+  set_usage();
+
   if (argc < 4)
     usage(0);
 
@@ -68,14 +71,14 @@ main(int argc,
     if (!parse_int(argv[4], maxlen) || (maxlen < 4))
       mxerror(_("Max line length must be >= 4.\n\n"));
   } else if ((argc > 5) || ((argc > 4) && (mode == 'd')))
-    usage(1);
+    usage(2);
 
   maxlen = ((maxlen + 3) / 4) * 4;
 
   try {
     in = new mm_file_io_c(argv[2]);
     if (mode != 'e')
-      in = new mm_text_io_c(in);
+      intext = new mm_text_io_c(in);
   } catch(...) {
     mxerror(_("The file '%s' could not be opened for reading (%d, %s).\n"),
             argv[2], errno, strerror(errno));
