@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: dts_common.cpp 2802 2005-02-20 18:40:37Z mosu $
+   $Id: dts_common.cpp 3064 2005-09-03 12:32:53Z mosu $
 
    helper function for DTS data
 
@@ -86,9 +86,9 @@ enum source_pcm_resolution {
 };
 
 int
-find_dts_header(const unsigned char *buf,
-                unsigned int size,
-                struct dts_header_s *dts_header) {
+find_dts_header_internal(const unsigned char *buf,
+                         unsigned int size,
+                         struct dts_header_s *dts_header) {
 
   unsigned int size_to_search = size-15;
   if (size_to_search > size) {
@@ -115,31 +115,31 @@ find_dts_header(const unsigned char *buf,
 
   unsigned int t;
 
-  bc.get_bits(1, t);
+  t = bc.get_bit();
   dts_header->frametype = (t)? dts_header_s::FRAMETYPE_NORMAL :
     dts_header_s::FRAMETYPE_TERMINATION;
 
-  bc.get_bits(5, t);
+  t = bc.get_bits(5);
   dts_header->deficit_sample_count = (t+1) % 32;
 
-  bc.get_bits(1, t);
+  t = bc.get_bit();
   dts_header->crc_present = (t)? true : false;
 
-  bc.get_bits(7, t);
+  t = bc.get_bits(7);
   if (t < 5)  {
     mxwarn("DTS_Header problem: invalid number of blocks in frame\n");
     //return -1;
   }
   dts_header->num_pcm_sample_blocks = t + 1;
 
-  bc.get_bits(14, t);
+  t = bc.get_bits(14);
   if (t < 95) {
     mxwarn("DTS_Header problem: invalid frame bytes size\n");
     return -1;
   }
   dts_header->frame_byte_size = t+1;
 
-  bc.get_bits(6, t);
+  t = bc.get_bits(6);
   if (t >= 16) {
     dts_header->audio_channels = -1;
     dts_header->audio_channel_arrangement = "unknown (user defined)";
@@ -149,45 +149,45 @@ find_dts_header(const unsigned char *buf,
       channel_arrangements[t].description;
   }
 
-  bc.get_bits(4, t);
+  t = bc.get_bits(4);
   dts_header->core_sampling_frequency = core_samplefreqs[t];
   if (dts_header->core_sampling_frequency < 0) {
     mxwarn("DTS_Header problem: invalid core sampling frequency\n");
     return -1;
   }
 
-  bc.get_bits(5, t);
+  t = bc.get_bits(5);
   dts_header->transmission_bitrate = transmission_bitrates[t];
 
-  bc.get_bit(dts_header->embedded_down_mix);
-  bc.get_bit(dts_header->embedded_dynamic_range);
-  bc.get_bit(dts_header->embedded_time_stamp);
-  bc.get_bit(dts_header->auxiliary_data);
-  bc.get_bit(dts_header->hdcd_master);
+  dts_header->embedded_down_mix = bc.get_bit();
+  dts_header->embedded_dynamic_range = bc.get_bit();
+  dts_header->embedded_time_stamp = bc.get_bit();
+  dts_header->auxiliary_data = bc.get_bit();
+  dts_header->hdcd_master = bc.get_bit();
 
-  bc.get_bits(3, t);
+  t = bc.get_bits(3);
   dts_header->extension_audio_descriptor =
     (dts_header_s::extension_audio_descriptor_e)t;
 
-  bc.get_bit(dts_header->extended_coding);
+  dts_header->extended_coding = bc.get_bit();
 
-  bc.get_bit(dts_header->audio_sync_word_in_sub_sub);
+  dts_header->audio_sync_word_in_sub_sub = bc.get_bit();
 
-  bc.get_bits(2, t);
+  t = bc.get_bits(2);
   dts_header->lfe_type = (dts_header_s::lfe_type_e)t;
 
-  bc.get_bit(dts_header->predictor_history_flag);
+  dts_header->predictor_history_flag = bc.get_bit();
 
   if (dts_header->crc_present) {
-    bc.get_bits(16, t);
+    t = bc.get_bits(16);
     // unsigned short header_CRC_sum = t; // not used yet
   }
 
-  bc.get_bits(1, t);
+  t = bc.get_bit();
   dts_header->multirate_interpolator =
     (dts_header_s::multirate_interpolator_e)t;
 
-  bc.get_bits(4, t);
+  t = bc.get_bits(4);
   dts_header->encoder_software_revision = t;
   if (t > 7) {
     mxwarn("DTS_Header problem: encoded with an incompatible new "
@@ -195,10 +195,10 @@ find_dts_header(const unsigned char *buf,
     return -1;
   }
 
-  bc.get_bits(2, t);
+  t = bc.get_bits(2);
   dts_header->copy_history = t;
 
-  bc.get_bits(3, t);
+  t = bc.get_bits(3);
   switch (t) {
     case spr_16:
       dts_header->source_pcm_resolution = 16;
@@ -235,11 +235,11 @@ find_dts_header(const unsigned char *buf,
       return -1;
   }
 
-  bc.get_bit(dts_header->front_sum_difference);
+  dts_header->front_sum_difference = bc.get_bit();
 
-  bc.get_bit(dts_header->surround_sum_difference);
+  dts_header->surround_sum_difference = bc.get_bit();
 
-  bool out_of_data = !bc.get_bits(4, t);
+  t = bc.get_bits(4);
   if (dts_header->encoder_software_revision == 7) {
     dts_header->dialog_normalization_gain = -((int)t);
   } else if (dts_header->encoder_software_revision == 6) {
@@ -248,13 +248,20 @@ find_dts_header(const unsigned char *buf,
     dts_header->dialog_normalization_gain = 0;
   }
 
-  if (out_of_data) {
+  return offset;
+
+}
+
+int
+find_dts_header(const unsigned char *buf,
+                unsigned int size,
+                struct dts_header_s *dts_header) {
+  try {
+    return find_dts_header_internal(buf, size, dts_header);
+  } catch (...) {
     mxwarn("DTS_Header problem: not enough data to read header\n");
     return -1;
   }
-
-  return offset;
-
 }
 
 // ============================================================================
