@@ -6,16 +6,24 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: extern_data.cpp 2955 2005-06-12 09:31:11Z mosu $
+   $Id: extern_data.cpp 3222 2006-04-28 19:49:53Z mosu $
 
    Data for ComboBoxes in mmg and other occurences
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
+   Changes by Robert Millan <rmh@aybabtu.com>.
 */
 
 #include "os.h"
+#include "common.h"
 #include "extern_data.h"
 
+#if HAVE_MAGIC_H
+extern "C" {
+#include <magic.h>
+};
+#endif
+
 #ifndef NULL
 # define NULL 0
 #endif
@@ -2598,3 +2606,67 @@ const char *cctlds[] = {
   "zw",
   NULL
 };
+
+static string
+guess_mime_type_by_ext(string ext) {
+  vector<string> extensions;
+  int i, j;
+
+  /* chop off basename */
+  i = ext.rfind('.');
+  if (i < 0)
+    return "";
+  ext.erase(0, i + 1);
+  ext = downcase(ext);
+
+  for (i = 0; NULL != mime_types[i].name; i++) {
+    if (0 == mime_types[i].extensions[0])
+      continue;
+
+    extensions = split(mime_types[i].extensions, " ");
+    for (j = 0; j < extensions.size(); j++)
+      if (downcase(extensions[j]) == ext)
+        return mime_types[i].name;
+  }
+
+  return "";
+}
+
+string
+guess_mime_type(string ext,
+                bool is_file) {
+#if HAVE_MAGIC_H
+  string ret;
+  magic_t m;
+
+  if (!is_file)
+    return guess_mime_type_by_ext(ext);
+
+  m = magic_open(MAGIC_MIME | MAGIC_SYMLINK);
+  magic_load(m, NULL);
+  ret = magic_file(m, ext.c_str());
+  magic_close(m);
+
+  if (ret == "")
+    return guess_mime_type_by_ext(ext);
+  else {
+    int idx = ret.find(';');
+    if (-1 != idx)
+      ret.erase(idx);
+    return ret;
+  }
+#else
+  return guess_mime_type_by_ext(ext);
+#endif
+}
+
+bool
+is_valid_cctld(const string &s) {
+  int i;
+
+  for (i = 0; NULL != cctlds[i]; ++i)
+    if (s == cctlds[i])
+      return true;
+
+  return false;
+}
