@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: hacks.cpp 3031 2005-08-21 13:04:48Z mosu $
+   $Id: hacks.cpp 3414 2007-01-03 10:23:22Z mosu $
 
    hacks :)
 
@@ -36,9 +36,10 @@ static const char *mosu_hacks[] = {
   ENGAGE_NO_VARIABLE_DATA,
   ENGAGE_NO_DEFAULT_HEADER_VALUES,
   ENGAGE_FORCE_PASSTHROUGH_PACKETIZER,
-  ENGAGE_AVC_USE_BFRAMES,
   ENGAGE_WRITE_HEADERS_TWICE,
-  ENGAGE_ALLOW_AVC_IN_VFW_MODE,
+  ENGAGE_KEEP_BITSTREAM_AR_INFO,
+  ENGAGE_USE_SIMPLE_BLOCK,
+  ENGAGE_OLD_AAC_CODECID,
   NULL
 };
 static vector<string> engaged_hacks;
@@ -47,7 +48,7 @@ bool
 hack_engaged(const string &hack) {
   vector<string>::const_iterator hit;
 
-  foreach(hit, engaged_hacks)
+  mxforeach(hit, engaged_hacks)
     if (*hit == hack)
       return true;
 
