@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tracks.cpp 2942 2005-06-05 11:01:39Z mosu $
+   $Id: tracks.cpp 3461 2007-01-12 14:44:59Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -23,7 +23,7 @@
 #include <time.h>
 
 #if defined(COMP_MSC)
-#include <assert.h>
+#include <cassert>
 #else
 #include <unistd.h>
 #endif
@@ -92,7 +92,7 @@ create_extractors(KaxTracks &kax_tracks,
     extractor = NULL;
     for (k = 0; k < extractors.size(); k++)
       if (extractors[k]->tid == tnum) {
-        mxwarn("More than one track with the track number %lld found.\n",
+        mxwarn("More than one track with the track number " LLD " found.\n",
                tnum);
         extractor = extractors[k];
         break;
@@ -113,11 +113,12 @@ create_extractors(KaxTracks &kax_tracks,
     // Let's find the codec ID and create an extractor for it.
     codec_id = kt_get_codec_id(track);
     if (codec_id.empty())
-      mxerror("The track number %lld does not have a valid CodecID.\n", tnum);
+      mxerror("The track number " LLD " does not have a valid CodecID.\n",
+              tnum);
 
     extractor = xtr_base_c::create_extractor(codec_id, tnum, *tspec);
     if (NULL == extractor)
-      mxerror("Extraction of track number %lld with the CodecID '%s' is "
+      mxerror("Extraction of track number " LLD " with the CodecID '%s' is "
               "not supported.\n", tnum, codec_id.c_str());
 
     // Has there another file been requested with the same name?
@@ -134,8 +135,9 @@ create_extractors(KaxTracks &kax_tracks,
     // We're done.
     extractors.push_back(extractor);
 
-    mxinfo("Extracting track %lld with the CodecID '%s' to the file '%s'.\n",
-           tnum, codec_id.c_str(), tspec->out_name);
+    mxinfo("Extracting track " LLD " with the CodecID '%s' to the file '%s'. "
+           "Container format: %s\n", tnum, codec_id.c_str(), tspec->out_name,
+           extractor->get_container_name());
   }
 
   // Signal that all headers have been taken care of.
@@ -157,8 +159,9 @@ handle_blockgroup(KaxBlockGroup &blockgroup,
 
   // Only continue if this block group actually contains a block.
   block = FINDFIRST(&blockgroup, KaxBlock);
-  if (NULL == block)
+  if ((NULL == block) || (0 == block->NumberFrames()))
     return;
+
   block->SetParent(cluster);
 
   // Do we need this block group?
@@ -193,9 +196,71 @@ handle_blockgroup(KaxBlockGroup &blockgroup,
   // Any block additions present?
   kadditions = FINDFIRST(&blockgroup, KaxBlockAdditions);
 
-  // Pass the block to the extractor.
-  extractor->handle_block(*block, kadditions, block->GlobalTimecode(),
-                          duration, bref, fref);
+  if (0 > duration)
+    duration = extractor->default_duration * block->NumberFrames();
+
+  for (i = 0; i < block->NumberFrames(); i++) {
+    int64_t this_timecode, this_duration;
+
+    if (0 > duration) {
+      this_timecode = block->GlobalTimecode();
+      this_duration = duration;
+    } else {
+      this_timecode = block->GlobalTimecode() + i * duration /
+        block->NumberFrames();
+      this_duration = duration / block->NumberFrames();
+    }
+
+    DataBuffer &data = block->GetBuffer(i);
+    memory_cptr frame(new memory_c(data.Buffer(), data.Size(), false));
+    extractor->handle_frame(frame, kadditions, this_timecode, this_duration,
+                            bref, fref, false, false, true);
+  }
+}
+
+static void
+handle_simpleblock(KaxSimpleBlock &simpleblock,
+                   KaxCluster &cluster) {
+  xtr_base_c *extractor;
+  int64_t duration;
+  size_t i;
+
+  if (0 == simpleblock.NumberFrames())
+    return;
+
+  simpleblock.SetParent(cluster);
+
+  // Do we need this block group?
+  extractor = NULL;
+  for (i = 0; i < extractors.size(); i++)
+    if (simpleblock.TrackNum() == extractors[i]->tid) {
+      extractor = extractors[i];
+      break;
+    }
+
+  if (NULL == extractor)
+    return;
+
+  duration = extractor->default_duration * simpleblock.NumberFrames();
+
+  for (i = 0; i < simpleblock.NumberFrames(); i++) {
+    int64_t this_timecode, this_duration;
+
+    if (0 > duration) {
+      this_timecode = simpleblock.GlobalTimecode();
+      this_duration = duration;
+    } else {
+      this_timecode = simpleblock.GlobalTimecode() + i * duration /
+        simpleblock.NumberFrames();
+      this_duration = duration / simpleblock.NumberFrames();
+    }
+
+    DataBuffer &data = simpleblock.GetBuffer(i);
+    memory_cptr frame(new memory_c(data.Buffer(), data.Size(), false));
+    extractor->handle_frame(frame, NULL, this_timecode, this_duration,
+                            -1, -1, simpleblock.IsKeyframe(),
+                            simpleblock.IsDiscardable(), false);
+  }
 }
 
 static void
@@ -251,7 +316,7 @@ write_all_cuesheets(KaxChapters &chapters,
         mxerror(_("The file '%s' could not be opened for writing (%s).\n"),
                 cue_file_name.c_str(), strerror(errno));
       }
-      mxinfo(_("The CUE sheet for track %lld will be written to '%s'.\n"),
+      mxinfo(_("The CUE sheet for track " LLD " will be written to '%s'.\n"),
              tspecs[i].tid, cue_file_name.c_str());
       write_cuesheet(file_name.c_str(), chapters, tags, tspecs[i].tuid, *out);
       delete out;
@@ -362,7 +427,7 @@ extract_tracks(const char *file_name,
             KaxTimecodeScale &ktc_scale = *static_cast<KaxTimecodeScale *>(l2);
             ktc_scale.ReadData(es->I_O());
             tc_scale = uint64(ktc_scale);
-            show_element(l2, 2, _("Timecode scale: %llu"), tc_scale);
+            show_element(l2, 2, _("Timecode scale: " LLU), tc_scale);
           } else
             l2->SkipData(*es, l2->Generic().Context);
 
@@ -387,7 +452,7 @@ extract_tracks(const char *file_name,
 
       } else if ((EbmlId(*l1) == KaxTracks::ClassInfos.GlobalId) &&
                  !tracks_found) {
-        
+
         // Yep, we've found our KaxTracks element. Now find all tracks
         // contained in this segment.
         show_element(l1, 1, _("Segment tracks"));
@@ -426,6 +491,13 @@ extract_tracks(const char *file_name,
             handle_blockgroup(*static_cast<KaxBlockGroup *>(l2), *cluster,
                               tc_scale);
 
+          } else if (EbmlId(*l2) == KaxSimpleBlock::ClassInfos.GlobalId) {
+            show_element(l2, 2, _("SimpleBlock"));
+
+            l2->Read(*es, KaxSimpleBlock::ClassInfos.Context, upper_lvl_el, l3,
+                     true);
+            handle_simpleblock(*static_cast<KaxSimpleBlock *>(l2), *cluster);
+
           } else
             l2->SkipData(*es, l2->Generic().Context);
 
