@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_ogg.h 2765 2005-02-05 17:36:14Z mosu $
+   $Id: xtr_ogg.h 3461 2007-01-12 14:44:59Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -32,8 +32,7 @@ class xtr_oggbase_c: public xtr_base_c {
 public:
   ogg_stream_state os;
   int packetno;
-  unsigned char *buffered_data;
-  int buffered_size;
+  memory_cptr buffered_data;
   int sfreq;
   int64_t previous_end;
 
@@ -41,9 +40,10 @@ public:
   xtr_oggbase_c(const string &_codec_id, int64_t _tid, track_spec_t &tspec);
 
   virtual void create_file(xtr_base_c *_master, KaxTrackEntry &track);
-  virtual void handle_block(KaxBlock &block, KaxBlockAdditions *additions,
+  virtual void handle_frame(memory_cptr &frame, KaxBlockAdditions *additions,
                             int64_t timecode, int64_t duration, int64_t bref,
-                            int64_t fref);
+                            int64_t fref, bool keyframe, bool discardable,
+                            bool references_valid);
   virtual void finish_file();
 
   virtual void write_pages();
@@ -55,6 +55,10 @@ public:
   xtr_oggflac_c(const string &_codec_id, int64_t _tid, track_spec_t &tspec);
 
   virtual void create_file(xtr_base_c *_master, KaxTrackEntry &track);
+
+  virtual const char *get_container_name() {
+    return "Ogg (FLAC in Ogg)";
+  };
 };
 
 class xtr_oggvorbis_c: public xtr_oggbase_c {
@@ -62,6 +66,10 @@ public:
   xtr_oggvorbis_c(const string &_codec_id, int64_t _tid, track_spec_t &tspec);
 
   virtual void create_file(xtr_base_c *_master, KaxTrackEntry &track);
+
+  virtual const char *get_container_name() {
+    return "Ogg (Vorbis in Ogg)";
+  };
 };
 
 #endif
