@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: mkvmerge.cpp 3022 2005-08-15 20:47:56Z mosu $
+   \version $Id: mkvmerge.cpp 3421 2007-01-04 08:07:17Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -18,6 +18,7 @@
 
 #include <errno.h>
 #include <ctype.h>
+#include <pcrecpp.h>
 #if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
 #include <signal.h>
 #endif
@@ -75,6 +76,7 @@ file_type_t file_types[] =
    {"ac3 ", FILE_TYPE_AC3, "A/52 (aka AC3)"},
    {"avi ", FILE_TYPE_AVI, "AVI (Audio/Video Interleaved)"},
    {"dts ", FILE_TYPE_DTS, "DTS (Digital Theater System)"},
+   {"h264", FILE_TYPE_AVC_ES, "AVC/h.264 elementary streams"},
    {"mp2 ", FILE_TYPE_MP3, "MPEG-1 layer II audio (CBR and VBR/ABR)"},
    {"mp3 ", FILE_TYPE_MP3, "MPEG-1 layer III audio (CBR and VBR/ABR)"},
    {"mkv ", FILE_TYPE_MATROSKA, "general Matroska files"},
@@ -102,10 +104,15 @@ file_type_t file_types[] =
    {"    ", -1,      "FLAC"},
 #endif
    {"    ", -1,      "MP3 audio"},
+   {"    ", -1,      "RealAudio and RealVideo"},
    {"    ", -1,      "simple text subtitles"},
+   {"    ", -1,      "text subtitles (SRT, SSA/ASS)"},
+   {"    ", -1,      "TTA lossless audio"},
    {"    ", -1,      "uncompressed PCM audio"},
-   {"    ", -1,      "Video (not MPEG1/2)"},
+   {"    ", -1,      "video (MPEG 1, MPEG 2, MPEG 4 part 2 and 10)"},
+   {"    ", -1,      "VobSub subtitles"},
    {"    ", -1,      "Vorbis audio"},
+   {"    ", -1,      "WAVPACK lossless audio"},
    {NULL,  -1,      NULL}};
 
 /** \brief Outputs usage information
@@ -167,6 +174,8 @@ set_usage() {
     "                           Description for the following attachment.\n"
     "  --attachment-mime-type <mime type>\n"
     "                           Mime type for the following attachment.\n"
+    "  --attachment-name <name> The name should be stored for the \n"
+    "                           following attachment.\n"
     "  --attach-file <file>     Creates a file attachment inside the\n"
     "                           Matroska file.\n"
     "  --attach-file-once <file>\n"
@@ -212,8 +221,13 @@ set_usage() {
     "  -t, --tags <TID:file>    Read tags for the track from a XML file.\n"
     "  --aac-is-sbr <TID>       Track with the ID is HE-AAC/AAC+/SBR-AAC.\n"
     "  --timecodes <TID:file>   Read the timecodes to be used from a file.\n"
-    "  --default-duration <TID:Xms|us|ns>\n"
+    "  --default-duration <TID:Xs|ms|us|ns|fps>\n"
     "                           Force the default duration of a track to X.\n"
+    "                           X can be a floating point number or a fration."
+    "\n"
+    "  --nalu-size-length <TID:n>\n"
+    "                           Force the NALU size length to n bytes with\n"
+    "                           2 <= n <= 4.\n"
     "\n Options that only apply to video tracks:\n"
     "  -f, --fourcc <FOURCC>    Forces the FourCC to the specified value.\n"
     "                           Works only for video tracks.\n"
@@ -229,6 +243,10 @@ set_usage() {
     "                           Explicitely set the display dimensions.\n"
     "  --cropping <TID:left,top,right,bottom>\n"
     "                           Sets the cropping parameters.\n"
+    "  --stereo-mode <TID:n|none|left|right|both>\n"
+    "                           Sets the stereo mode parameter. It can\n"
+    "                           either be a numer 0 - 3 or one of the\n"
+    "                           keywords 'none', 'right', 'left' or 'both'.\n"
     "\n Options that only apply to text subtitle tracks:\n"
     "  --sub-charset <TID:charset>\n"
     "                           Sets the charset the text subtitles are\n"
@@ -275,7 +293,7 @@ print_capabilities() {
 #if defined(HAVE_BZLIB_H)
   mxinfo("BZ2\n");
 #endif
-#if defined(HAVE_LZO1X_H)
+#if defined(HAVE_LZO)
   mxinfo("LZO\n");
 #endif
 #if defined(HAVE_FLAC_FORMAT_H)
@@ -364,42 +382,68 @@ identify(const string &filename) {
 
 /** \brief Parse a number postfixed with a time-based unit
 
-   This function parsers a number that is postfixed with one of the four
-   units 's', 'ms', 'us' or 'ns'. It returns a number of nanoseconds.
+   This function parsers a number that is postfixed with one of the
+   four units 's', 'ms', 'us' or 'ns'. If the postfix is 'fps' then
+   this means 'frames per second'. It returns a number of nanoseconds.
 */
 int64_t
 parse_number_with_unit(const string &s,
                        const string &subject,
-                       const string &argument) {
-  string unit;
-  int64_t value, multiplier, unit_length;
+                       const string &argument,
+                       string display_s = "") {
+  pcrecpp::RE_Options opt_caseless;
+  opt_caseless.set_caseless(true);
+  pcrecpp::RE re1("(-?\\d+\\.?\\d*)(s|ms|us|ns|fps)?", opt_caseless);
+  pcrecpp::RE re2("(-?\\d+)/(-?\\d+)(s|ms|us|ns|fps)?", opt_caseless);
+  string unit, s_n, s_d;
+  int64_t n, d, multiplier;
+  double d_value = 0.0;
+  bool is_fraction = false;
 
-  if (s.length() < 3)
+  if (display_s == "")
+    display_s = s;
+
+  if (re1.FullMatch(s, &s_n, &unit)) {
+    parse_double(s_n, d_value);
+    d = 1;
+
+  } else if (re2.FullMatch(s, &s_n, &s_d, &unit)) {
+    parse_int(s_n, n);
+    parse_int(s_d, d);
+    is_fraction = true;
+
+  } else
     mxerror(_("'%s' is not a valid %s in '%s %s'.\n"),
-            s.c_str(), subject.c_str(), argument.c_str(), s.c_str());
-
-  unit = s.substr(s.length() - 2, 2);
+            s.c_str(), subject.c_str(), argument.c_str(), display_s.c_str());
 
   multiplier = 1000000000;
-  unit_length = 2;
+
+  unit = downcase(unit);
+
   if (unit == "ms")
     multiplier = 1000000;
   else if (unit == "us")
     multiplier = 1000;
   else if (unit == "ns")
     multiplier = 1;
-  else if (unit.substr(1, 1) == "s")
-    unit_length = 1;
-  else
+  else if (unit == "fps") {
+    if (is_fraction)
+      return 1000000000ll * d / n;
+    else if (29.97 == d_value)
+      return (int64_t)(100100000.0 / 3.0);
+    else if (23.976 == d_value)
+      return 40040000;
+    else
+      return (int64_t)(1000000000.0 / d_value);
+  } else if (unit != "s")
     mxerror(_("'%s' does not contain a valid unit ('s', 'ms', 'us' or 'ns') "
-              "in '%s %s'.\n"), s.c_str(), argument.c_str(), s.c_str());
+              "in '%s %s'.\n"), s.c_str(), argument.c_str(),
+            display_s.c_str());
 
-  if (!parse_int(s.substr(0, s.length() - unit_length), value))
-    mxerror(_("'%s' does not contain a valid number in front of the unit in "
-              "'%s %s'.\n"), s.c_str(), argument.c_str(), s.c_str());
-  value *= multiplier;
-
-  return value;
+  if (is_fraction)
+    return multiplier * n / d;
+  else
+    return (int64_t)(multiplier * d_value);
 }
 
 /** \brief Parse tags and add them to the list of all tags
@@ -470,6 +514,7 @@ parse_sync(string s,
   string orig, linear, div;
   int idx;
   double d1, d2;
+  int64_t id;
 
   // Extract the track number.
   orig = s;
@@ -478,7 +523,8 @@ parse_sync(string s,
     mxerror(_("Invalid sync option. No track ID specified in '%s %s'.\n"),
             opt.c_str(), s.c_str());
 
-  if (!parse_int(parts[0], async.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("Invalid track ID specified in '%s %s'.\n"), opt.c_str(),
             s.c_str());
 
@@ -515,7 +561,7 @@ parse_sync(string s,
     async.linear = 1.0;
   async.displacement = atoi(s.c_str());
 
-  ti.audio_syncs.push_back(async);
+  ti.audio_syncs[id] = async;
 }
 
 /** \brief Parse the \c --aspect-ratio argument
@@ -533,6 +579,7 @@ parse_aspect_ratio(const string &s,
   vector<string> parts;
   display_properties_t dprop;
   const char *msg;
+  int64_t id;
 
   if (is_factor)
     msg = _("Aspect ratio factor");
@@ -544,7 +591,8 @@ parse_aspect_ratio(const string &s,
   if (parts.size() != 2)
     mxerror(_("%s: missing track ID in '%s %s'.\n"), msg, opt.c_str(),
             s.c_str());
-  if (!parse_int(parts[0], dprop.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("%s: invalid track ID in '%s %s'.\n"), msg, opt.c_str(),
             s.c_str());
   dprop.width = -1;
@@ -555,7 +603,7 @@ parse_aspect_ratio(const string &s,
     idx = parts[1].find(':');
   if (idx < 0) {
     dprop.aspect_ratio = strtod(parts[1].c_str(), NULL);
-    ti.display_properties.push_back(dprop);
+    ti.display_properties[id] = dprop;
     return;
   }
 
@@ -576,7 +624,7 @@ parse_aspect_ratio(const string &s,
             s.c_str());
 
   dprop.aspect_ratio = w / h;
-  ti.display_properties.push_back(dprop);
+  ti.display_properties[id] = dprop;
 }
 
 /** \brief Parse the \c --display-dimensions argument
@@ -589,6 +637,7 @@ parse_display_dimensions(const string s,
   vector<string> parts, dims;
   int w, h;
   display_properties_t dprop;
+  int64_t id;
 
   parts = split(s, ":", 2);
   strip(parts);
@@ -598,7 +647,8 @@ parse_display_dimensions(const string s,
             s.c_str());
 
   dims = split(parts[1], "x", 2);
-  if ((dims.size() != 2) || !parse_int(parts[0], dprop.id) ||
+  id = 0;
+  if ((dims.size() != 2) || !parse_int(parts[0], id) ||
       !parse_int(dims[0], w) || !parse_int(dims[1], h) ||
       (w <= 0) || (h <= 0))
     mxerror(_("Display dimensions: not given in the form "
@@ -609,7 +659,7 @@ parse_display_dimensions(const string s,
   dprop.width = w;
   dprop.height = h;
 
-  ti.display_properties.push_back(dprop);
+  ti.display_properties[id] = dprop;
 }
 
 /** \brief Parse the \c --cropping argument
@@ -622,13 +672,15 @@ parse_cropping(const string &s,
                track_info_c &ti) {
   pixel_crop_t crop;
   vector<string> v;
+  int64_t id;
 
   v = split(s, ":");
   if (v.size() != 2)
     mxerror(_("Cropping parameters: not given in the form "
               "<TID>:<left>,<top>,<right>,<bottom> e.g. 0:10,5,10,5 "
               "(argument was '%s').\n"), s.c_str());
-  if (!parse_int(v[0], crop.id))
+  id = 0;
+  if (!parse_int(v[0], id))
     mxerror(_("Cropping parameters: not given in the form "
               "<TID>:<left>,<top>,<right>,<bottom> e.g. 0:10,5,10,5 "
               "(argument was '%s').\n"), s.c_str());
@@ -646,7 +698,46 @@ parse_cropping(const string &s,
               "<TID>:<left>,<top>,<right>,<bottom> e.g. 0:10,5,10,5 "
               "(argument was '%s').\n"), s.c_str());
 
-  ti.pixel_crop_list.push_back(crop);
+  ti.pixel_crop_list[id] = crop;
+}
+
+/** \brief Parse the \c --stereo-mode argument
+
+   The argument must either be a number between 0 and 3 or
+   one of the keywords \c 'none', \c 'left', \c 'right' or \c 'both'.
+*/
+static void
+parse_stereo_mode(const string &s,
+                  track_info_c &ti) {
+  static const char * const keywords[] = {
+    "none", "right", "left", "both", NULL
+  };
+  static const char *errmsg =
+    _("Stereo mode parameter: not given in the form "
+      "<TID>:<n|keyword> where n is a number between 0 and 3 "
+      "or one of the keywords 'none', 'right', 'left', 'both' "
+      "(argument was '%s').\n");
+  int64_t i, id;
+  vector<string> v;
+
+  v = split(s, ":");
+  if (v.size() != 2)
+    mxerror(errmsg, s.c_str());
+
+  id = 0;
+  if (!parse_int(v[0], id))
+    mxerror(errmsg, s.c_str());
+
+  for (i = 0; NULL != keywords[i]; ++i)
+    if (v[1] == keywords[i]) {
+      ti.stereo_mode_list[id] = (stereo_mode_e)i;
+      return;
+    }
+
+  if (!parse_int(v[1], i) || (i < 0) || (i > STEREO_MODE_BOTH))
+    mxerror(errmsg, s.c_str());
+
+  ti.stereo_mode_list[id] = (stereo_mode_e)i;
 }
 
 /** \brief Parse the duration formats to \c --split
@@ -687,7 +778,7 @@ parse_split_timecodes(const string &arg) {
     s.erase(0, 10);
 
   timecodes = split(s, ",");
-  foreach(timecode, timecodes) {
+  mxforeach(timecode, timecodes) {
     if (!parse_timecode(*timecode, split_after))
       mxerror(_("Invalid time for '--split' in '--split %s'. Additional error "
                 "message: %s.\n"), arg.c_str(), timecode_parser_error.c_str());
@@ -802,7 +893,7 @@ parse_delay(const string &s,
             track_info_c &ti) {
   string unit;
   vector<string> parts;
-  packet_delay_t delay;
+  int64_t id;
 
   // Extract the track number.
   parts = split(s, ":", 2);
@@ -811,12 +902,12 @@ parse_delay(const string &s,
     mxerror(_("Invalid delay option. No track ID specified in "
               "'--delay %s'.\n"), s.c_str());
 
-  if (!parse_int(parts[0], delay.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("Invalid track ID specified in '--delay %s'.\n"), s.c_str());
 
-  delay.delay = parse_number_with_unit(parts[1], "delay", "--delay");
-
-  ti.packet_delays.push_back(delay);
+  ti.packet_delays[id] = parse_number_with_unit(parts[1], "delay", "--delay",
+                                                s);
 }
 
 /** \brief Parse the \c --cues argument
@@ -827,7 +918,7 @@ static void
 parse_cues(const string &s,
            track_info_c &ti) {
   vector<string> parts;
-  cue_creation_t cues;
+  int64_t id;
 
   // Extract the track number.
   parts = split(s, ":", 2);
@@ -836,7 +927,8 @@ parse_cues(const string &s,
     mxerror(_("Invalid cues option. No track ID specified in '--cues %s'.\n"),
             s.c_str());
 
-  if (!parse_int(parts[0], cues.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("Invalid track ID specified in '--cues %s'.\n"), s.c_str());
 
   if (parts[1].size() == 0)
@@ -844,15 +936,13 @@ parse_cues(const string &s,
             s.c_str());
 
   if (parts[1] == "all")
-    cues.cues = CUE_STRATEGY_ALL;
+    ti.cue_creations[id] = CUE_STRATEGY_ALL;
   else if (parts[1] == "iframes")
-    cues.cues = CUE_STRATEGY_IFRAMES;
+    ti.cue_creations[id] = CUE_STRATEGY_IFRAMES;
   else if (parts[1] == "none")
-    cues.cues = CUE_STRATEGY_NONE;
+    ti.cue_creations[id] = CUE_STRATEGY_NONE;
   else
     mxerror(_("'%s' is an unsupported argument for --cues.\n"), s.c_str());
-
-  ti.cue_creations.push_back(cues);
 }
 
 /** \brief Parse the \c --compression argument
@@ -863,7 +953,7 @@ static void
 parse_compression(const string &s,
                   track_info_c &ti) {
   vector<string> parts;
-  compression_method_t compression;
+  int64_t id;
 
   // Extract the track number.
   parts = split(s, ":", 2);
@@ -872,7 +962,8 @@ parse_compression(const string &s,
     mxerror(_("Invalid compression option. No track ID specified in "
               "'--compression %s'.\n"), s.c_str());
 
-  if (!parse_int(parts[0], compression.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("Invalid track ID specified in '--compression %s'.\n"),
             s.c_str());
 
@@ -880,25 +971,25 @@ parse_compression(const string &s,
     mxerror(_("Invalid compression option specified in '--compression %s'.\n"),
             s.c_str());
 
-  compression.method = COMPRESSION_UNSPECIFIED;
+  ti.compression_list[id] = COMPRESSION_UNSPECIFIED;
   parts[1] = downcase(parts[1]);
-#ifdef HAVE_LZO1X_H
+#ifdef HAVE_LZO
   if ((parts[1] == "lzo") || (parts[1] == "lzo1x"))
-    compression.method = COMPRESSION_LZO;
+    ti.compression_list[id] = COMPRESSION_LZO;
 #endif
   if (parts[1] == "zlib")
-    compression.method = COMPRESSION_ZLIB;
+    ti.compression_list[id] = COMPRESSION_ZLIB;
 #ifdef HAVE_BZLIB_H
   if ((parts[1] == "bz2") || (parts[1] == "bzlib"))
-    compression.method = COMPRESSION_BZ2;
+    ti.compression_list[id] = COMPRESSION_BZ2;
 #endif
+  if ((parts[1] == "mpeg4_p2") || (parts[1] == "mpeg4p2"))
+    ti.compression_list[id] = COMPRESSION_MPEG4_P2;
   if (parts[1] == "none")
-    compression.method = COMPRESSION_NONE;
-  if (compression.method == COMPRESSION_UNSPECIFIED)
+    ti.compression_list[id] = COMPRESSION_NONE;
+  if (ti.compression_list[id] == COMPRESSION_UNSPECIFIED)
     mxerror(_("'%s' is an unsupported argument for --compression. Available "
               "compression methods are 'none' and 'zlib'.\n"), s.c_str());
-
-  ti.compression_list.push_back(compression);
 }
 
 /** \brief Parse the argument for a couple of options
@@ -908,12 +999,13 @@ parse_compression(const string &s,
 */
 static void
 parse_language(const string &s,
-               language_t &lang,
+               map<int64_t, string> &storage,
                const string &opt,
                const char *topic,
                bool check,
                bool empty_ok = false) {
   vector<string> parts;
+  int64_t id;
 
   // Extract the track number.
   parts = split(s, ":", 2);
@@ -928,29 +1020,29 @@ parse_language(const string &s,
     parts.push_back("");
   }
 
-  if (!parse_int(parts[0], lang.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("Invalid track ID specified in '--%s %s'.\n"), opt.c_str(),
             s.c_str());
 
   if (check) {
+    int index;
+
     if (parts[1].size() == 0)
       mxerror(_("Invalid %s specified in '--%s %s'.\n"), topic, opt.c_str(),
               s.c_str());
 
-    if (!is_valid_iso639_2_code(parts[1].c_str())) {
-      const char *iso639_2;
+    index = map_to_iso639_2_code(parts[1].c_str());
+    if (-1 == index)
+      mxerror(_("'%s' is neither a valid ISO639-2 nor a valid ISO639-1 code."
+                " See 'mkvmerge --list-languages' for a list of all "
+                "languages and their respective ISO639-2 codes.\n"),
+              parts[1].c_str());
 
-      iso639_2 = map_iso639_1_to_iso639_2(parts[1].c_str());
-      if (iso639_2 == NULL)
-        mxerror(_("'%s' is neither a valid ISO639-2 nor a valid ISO639-1 code."
-                  " See 'mkvmerge --list-languages' for a list of all "
-                  "languages and their respective ISO639-2 codes.\n"),
-                parts[1].c_str());
-      parts[1] = iso639_2;
-    }
+    parts[1] = iso639_languages[index].iso639_2_code;
   }
 
-  lang.language = parts[1];
+  storage[id] = parts[1];
 }
 
 /** \brief Parse the \c --subtitle-charset argument
@@ -961,7 +1053,7 @@ static void
 parse_sub_charset(const string &s,
                   track_info_c &ti) {
   vector<string> parts;
-  subtitle_charset_t sub_charset;
+  int64_t id;
 
   // Extract the track number.
   parts = split(s, ":", 2);
@@ -970,7 +1062,8 @@ parse_sub_charset(const string &s,
     mxerror(_("Invalid sub charset option. No track ID specified in "
               "'--sub-charset %s'.\n"), s.c_str());
 
-  if (!parse_int(parts[0], sub_charset.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("Invalid track ID specified in '--sub-charset %s'.\n"),
             s.c_str());
 
@@ -978,8 +1071,7 @@ parse_sub_charset(const string &s,
     mxerror(_("Invalid sub charset specified in '--sub-charset %s'.\n"),
             s.c_str());
 
-  sub_charset.charset = parts[1];
-  ti.sub_charsets.push_back(sub_charset);
+  ti.sub_charsets[id] = parts[1];
 }
 
 /** \brief Parse the \c --tags argument
@@ -991,7 +1083,7 @@ parse_tags(const string &s,
            const string &opt,
            track_info_c &ti) {
   vector<string> parts;
-  tags_t tags;
+  int64_t id;
 
   // Extract the track number.
   parts = split(s, ":", 2);
@@ -1000,7 +1092,8 @@ parse_tags(const string &s,
     mxerror(_("Invalid tags option. No track ID specified in '%s %s'.\n"),
             opt.c_str(), s.c_str());
 
-  if (!parse_int(parts[0], tags.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("Invalid track ID specified in '%s %s'.\n"), opt.c_str(),
             s.c_str());
 
@@ -1008,8 +1101,7 @@ parse_tags(const string &s,
     mxerror(_("Invalid tags file name specified in '%s %s'.\n"), opt.c_str(),
             s.c_str());
 
-  tags.file_name = safestrdup(parts[1]);
-  ti.all_tags.push_back(tags);
+  ti.all_tags[id] = parts[1];
 }
 
 /** \brief Parse the \c --fourcc argument
@@ -1022,22 +1114,22 @@ parse_fourcc(const string &s,
              track_info_c &ti) {
   vector<string> parts;
   string orig = s;
-  fourcc_t fourcc;
+  int64_t id;
 
   parts = split(s, ":", 2);
   strip(parts);
   if (parts.size() != 2)
     mxerror(_("FourCC: Missing track ID in '%s %s'.\n"), opt.c_str(),
             orig.c_str());
-  if (!parse_int(parts[0], fourcc.id))
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("FourCC: Invalid track ID in '%s %s'.\n"), opt.c_str(),
             orig.c_str());
   if (parts[1].size() != 4)
     mxerror(_("The FourCC must be exactly four characters long in '%s %s'."
               "\n"), opt.c_str(), orig.c_str());
-  memcpy(fourcc.fourcc, parts[1].c_str(), 4);
-  fourcc.fourcc[4] = 0;
-  ti.all_fourccs.push_back(fourcc);
+
+  ti.all_fourccs[id] = parts[1];
 }
 
 /** \brief Parse the argument for \c --track-order
@@ -1094,8 +1186,8 @@ parse_append_to(const string &s,
   append_mapping.clear();
   entries = split(s, ",");
   strip(entries);
-  foreach(entry, entries) {
-    if ((mxsscanf((*entry).c_str(), "%lld:%lld:%lld:%lld",
+  mxforeach(entry, entries) {
+    if ((mxsscanf((*entry).c_str(), "" LLD ":" LLD ":" LLD ":" LLD,
                   &mapping.src_file_id, &mapping.src_track_id,
                   &mapping.dst_file_id, &mapping.dst_track_id) != 4) ||
         (mapping.src_file_id < 0) || (mapping.src_track_id < 0) ||
@@ -1108,26 +1200,56 @@ parse_append_to(const string &s,
 
 /** \brief Parse the argument for \c --default-duration
 
-   The argument must be a tupel consisting of a track ID and the default
+   The argument must be a tuple consisting of a track ID and the default
    duration separated by a colon. The duration must be postfixed by 'ms',
-   'us' or 'ns'.
+   'us', 'ns' or 'fps' (see \c parse_number_with_unit).
 */
 static void
 parse_default_duration(const string &s,
                        track_info_c &ti) {
   vector<string> parts;
-  default_duration_t dd;
+  int64_t id;
 
   parts = split(s, ":");
   if (parts.size() != 2)
-    mxerror(_("'%s' is not a valid parts of track ID and default duration in "
+    mxerror(_("'%s' is not a valid tuple of track ID and default duration in "
               "'--default-duration %s'.\n"), s.c_str(), s.c_str());
-  if (!parse_int(parts[0], dd.id))
+
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("'%s' is not a valid track ID in '--default-duration %s'.\n"),
             parts[0].c_str(), s.c_str());
-  dd.default_duration = parse_number_with_unit(parts[1], "default duration",
-                                               "--default-duration");
-  ti.default_durations.push_back(dd);
+
+  ti.default_durations[id] =
+    parse_number_with_unit(parts[1], "default duration", "--default-duration");
+}
+
+/** \brief Parse the argument for \c --nalu-size-length
+
+   The argument must be a tuple consisting of a track ID and the NALU size
+   length, an integer between 2 and 4 inclusively.
+*/
+static void
+parse_nalu_size_length(const string &s,
+                       track_info_c &ti) {
+  vector<string> parts;
+  int64_t id, nalu_size_length;
+
+  parts = split(s, ":");
+  if (parts.size() != 2)
+    mxerror(_("'%s' is not a valid tuple of track ID and NALU size length in "
+              "'--nalu-size-length %s'.\n"), s.c_str(), s.c_str());
+
+  id = 0;
+  if (!parse_int(parts[0], id))
+    mxerror(_("'%s' is not a valid track ID in '--nalu-size-length %s'.\n"),
+            parts[0].c_str(), s.c_str());
+
+  if (!parse_int(parts[1], nalu_size_length) ||
+      (2 > nalu_size_length) || (4 < nalu_size_length))
+    mxerror(_("The NALU size length must be a number between 2 and 4 "
+              "inclusively in '--nalu-size-length %s'.\n"), s.c_str());
+  ti.nalu_size_lengths[id] = nalu_size_length;
 }
 
 /** \brief Parse the argument for \c --blockadd
@@ -1139,20 +1261,48 @@ static void
 parse_max_blockadd_id(const string &s,
                        track_info_c &ti) {
   vector<string> parts;
-  max_blockadd_id_t mbi;
+  int64_t id, max_blockadd_id;
 
   parts = split(s, ":");
   if (parts.size() != 2)
     mxerror(_("'%s' is not a valid parts of track ID and block additional in "
               "'--blockadd %s'.\n"), s.c_str(), s.c_str());
-  if (!parse_int(parts[0], mbi.id))
+
+  id = 0;
+  if (!parse_int(parts[0], id))
     mxerror(_("'%s' is not a valid track ID in '--blockadd %s'.\n"),
             parts[0].c_str(), s.c_str());
-  if (!parse_int(parts[1], mbi.max_blockadd_id) || (mbi.max_blockadd_id < 0))
+  max_blockadd_id = 0;
+  if (!parse_int(parts[1], max_blockadd_id) || (max_blockadd_id < 0))
     mxerror(_("'%s' is not a valid block additional max in '--blockadd %s'."
               "\n"),
             parts[1].c_str(), s.c_str());
-  ti.max_blockadd_ids.push_back(mbi);
+  ti.max_blockadd_ids[id] = max_blockadd_id;
+}
+
+/** \brief Parse the argument for \c --aac-is-sbr
+
+   The argument can either be just a number (the track ID) or a tupel
+   "trackID:number" where the second number is either "0" or "1". If only
+   a track ID is given then "number" is assumed to be "1".
+*/
+static void
+parse_aac_is_sbr(const string &s,
+                 track_info_c &ti) {
+  vector<string> parts;
+  int64_t id;
+
+  parts = split(s, ":", 2);
+
+  if (!parse_int(parts[0], id) || (id < 0))
+    mxerror(_("Invalid track ID specified in '--aac-is-sbr %s'.\n"),
+            s.c_str());
+
+  if ((parts.size() == 2) && (parts[1] != "0") && (parts[1] != "1"))
+    mxerror(_("Invalid boolean specified in '--aac-is-sbr %s'.\n"),
+            s.c_str());
+
+  ti.all_aac_is_sbr[id] = (1 == parts.size()) || (parts[1] == "1");
 }
 
 /** \brief Sets the priority mkvmerge runs with
@@ -1210,33 +1360,18 @@ set_process_priority(int priority) {
 }
 
 static string
-guess_mime_type(string file_name) {
-  vector<string> extensions;
-  int i, k;
+guess_mime_type_and_report(string file_name) {
+  string mime_type;
 
-  i = file_name.rfind('.');
-  if (i < 0)
-    mxerror(_("No MIME type has been set for the attachment '%s', and "
-              "the file name contains no extension. Therefore the MIME "
-              "type could not be guessed automatically.\n"),
-            file_name.c_str());
-  file_name.erase(0, i + 1);
-
-  for (i = 0; mime_types[i].name != NULL; i++) {
-    if (mime_types[i].extensions[0] == 0)
-      continue;
-
-    extensions = split(mime_types[i].extensions, " ");
-    for (k = 0; k < extensions.size(); k++)
-      if (!strcasecmp(extensions[k].c_str(), file_name.c_str())) {
-        mxinfo(_("Automatic MIME type recognition for '%s': %s\n"),
-               file_name.c_str(), mime_types[i].name);
-        return mime_types[i].name;
-      }
+  mime_type = guess_mime_type(file_name, true);
+  if (mime_type != "") {
+    mxinfo(_("Automatic MIME type recognition for '%s': %s\n"),
+           file_name.c_str(), mime_type.c_str());
+    return mime_type;
   }
 
   mxerror(_("No MIME type has been set for the attachment '%s', and "
-            "it could not be guessed based on its extension.\n"),
+            "it could not be guessed.\n"),
           file_name.c_str());
   return "";
 }
@@ -1260,7 +1395,7 @@ handle_segmentinfo(KaxInfo *kax_info_chap) {
   uid = FINDFIRST(kax_info_chap, KaxNextUID);
   if (NULL != uid)
     seguid_link_next = new bitvalue_c(*uid);
-  
+
   uid = FINDFIRST(kax_info_chap, KaxPrevUID);
   if (NULL != uid)
     seguid_link_previous = new bitvalue_c(*uid);
@@ -1287,11 +1422,13 @@ parse_args(vector<string> args) {
   int i;
   vector<string>::const_iterator sit;
   string this_arg, next_arg;
-  bool no_next_arg;
+  bool no_next_arg, inputs_found;
   int64_t id;
   attachment_t attachment;
   mm_io_c *io;
 
+  inputs_found = false;
+
   ti = new track_info_c;
 
   handle_common_cli_args(args, "");
@@ -1310,7 +1447,7 @@ parse_args(vector<string> args) {
   }
 
   // First parse options that either just print some infos and then exit.
-  foreach(sit, args) {
+  mxforeach(sit, args) {
     this_arg = *sit;
     if ((sit + 1) == args.end())
       next_arg = "";
@@ -1345,7 +1482,7 @@ parse_args(vector<string> args) {
   mxinfo("%s built on %s %s\n", version_info.c_str(), __DATE__, __TIME__);
 
   // Now parse options that are needed right at the beginning.
-  foreach(sit, args) {
+  mxforeach(sit, args) {
     this_arg = *sit;
     if ((sit + 1) == args.end())
       next_arg = "";
@@ -1376,7 +1513,7 @@ parse_args(vector<string> args) {
     usage(2);
   }
 
-  foreach(sit, args) {
+  mxforeach(sit, args) {
     this_arg = *sit;
     if ((sit + 1) == args.end())
       next_arg = "";
@@ -1539,27 +1676,46 @@ parse_args(vector<string> args) {
       attachment.mime_type = next_arg;
       sit++;
 
+    } else if (this_arg == "--attachment-name") {
+      if (no_next_arg)
+        mxerror(_("'--attachment-name' lacks the name.\n"));
+
+      if (attachment.stored_name != "")
+        mxwarn(_("More than one name was given for a single attachment. "
+                 "'%s' will be discarded and '%s' used instead.\n"),
+               attachment.stored_name.c_str(), next_arg.c_str());
+      attachment.stored_name = next_arg;
+      sit++;
+
     } else if ((this_arg == "--attach-file") ||
                (this_arg == "--attach-file-once")) {
       if (no_next_arg)
         mxerror(_("'%s' lacks the file name.\n"), this_arg.c_str());
 
+      try {
+        mm_file_io_c test(next_arg);
+      } catch (...) {
+        mxerror(_("The file '%s' cannot be attached because "
+                  "it does not exist or cannot be read.\n"),
+                next_arg.c_str());
+      }
+
       attachment.name = next_arg;
       if (attachment.mime_type == "")
-        attachment.mime_type = guess_mime_type(next_arg);
+        attachment.mime_type = guess_mime_type_and_report(next_arg);
 
       if (this_arg == "--attach-file")
         attachment.to_all_files = true;
       try {
         attachment.data = counted_ptr<buffer_t>(new buffer_t);
         io = new mm_file_io_c(attachment.name);
-        attachment.data->size = io->get_size();
-        if (0 == attachment.data->size)
+        attachment.data->m_size = io->get_size();
+        if (0 == attachment.data->m_size)
           mxerror("The size of attachment '%s' is 0.\n",
                   attachment.name.c_str());
-        attachment.data->buffer = (unsigned char *)
-          safemalloc(attachment.data->size);
-        io->read(attachment.data->buffer, attachment.data->size);
+        attachment.data->m_buffer = (unsigned char *)
+          safemalloc(attachment.data->m_size);
+        io->read(attachment.data->m_buffer, attachment.data->m_size);
         delete io;
       } catch (...) {
         mxerror(_("The attachment '%s' could not be read.\n"),
@@ -1571,6 +1727,8 @@ parse_args(vector<string> args) {
 
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--global-tags") {
       if (no_next_arg)
         mxerror(_("'--global-tags' lacks the file name.\n"));
@@ -1578,6 +1736,8 @@ parse_args(vector<string> args) {
       parse_and_add_tags(next_arg);
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--chapter-language") {
       if (no_next_arg)
         mxerror(_("'--chapter-language' lacks the language.\n"));
@@ -1590,13 +1750,15 @@ parse_args(vector<string> args) {
         mxerror(_("'--chapter-language' must be given before '--chapters' in "
                   "'--chapter-language %s'.\n"), next_arg.c_str());
 
-      if (!is_valid_iso639_2_code(next_arg.c_str()))
-        mxerror(_("'%s' is not a valid ISO639-2 language code. Run "
-                  "'mkvmerge --list-languages' for a complete list of all "
-                  "languages and their respective ISO639-2 codes.\n"),
-                next_arg.c_str());
+      i = map_to_iso639_2_code(next_arg.c_str());
+      if (-1 == i)
+        mxerror(_("'%s' is neither a valid ISO639-2 nor a valid ISO639-1 code "
+                  "in '--chapter-language %s'. See 'mkvmerge --list-languages'"
+                  "for a list of all languages and their respective ISO639-2 "
+                  "codes.\n"),
+                next_arg.c_str(), next_arg.c_str());
 
-      chapter_language = next_arg;
+      chapter_language = iso639_languages[i].iso639_2_code;
       sit++;
 
     } else if (this_arg == "--chapter-charset") {
@@ -1642,6 +1804,8 @@ parse_args(vector<string> args) {
                                     NULL, &tags_from_cue_chapters);
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--segmentinfo") {
       if (no_next_arg)
         mxerror(_("'--segmentinfo' lacks the file name.\n"));
@@ -1656,6 +1820,8 @@ parse_args(vector<string> args) {
 
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--no-chapters") {
       ti->no_chapters = true;
 
@@ -1773,6 +1939,13 @@ parse_args(vector<string> args) {
       parse_cropping(next_arg, *ti);
       sit++;
 
+    } else if (this_arg == "--stereo-mode") {
+      if (no_next_arg)
+        mxerror(_("'--stereo-mode' lacks the crop parameters.\n"));
+
+      parse_stereo_mode(next_arg, *ti);
+      sit++;
+
     } else if ((this_arg == "-y") || (this_arg == "--sync")) {
       if (no_next_arg)
         mxerror(_("'%s' lacks the audio delay.\n"), this_arg.c_str());
@@ -1806,25 +1979,25 @@ parse_args(vector<string> args) {
       sit++;
 
     } else if (this_arg == "--language") {
-      language_t lang;
-
       if (no_next_arg)
         mxerror(_("'--language' lacks its argument.\n"));
 
-      parse_language(next_arg, lang, "language", "language", true);
-      ti->languages.push_back(lang);
+      parse_language(next_arg, ti->languages, "language", "language", true);
       sit++;
 
     } else if (this_arg == "--default-language") {
       if (no_next_arg)
         mxerror(_("'--default-language' lacks its argument.\n"));
 
-      if (!is_valid_iso639_2_code(next_arg.c_str()))
-        mxerror(_("'%s' is not a valid ISO639-2 language code in "
-                  "'--default-language %s'. Run 'mkvmerge --list-languages' "
-                  "for a list of all languages and their respective "
-                  "ISO639-2 codes.\n"), next_arg.c_str(), next_arg.c_str());
-      default_language = next_arg;
+      i = map_to_iso639_2_code(next_arg.c_str());
+      if (-1 == i)
+        mxerror(_("'%s' is neither a valid ISO639-2 nor a valid ISO639-1 code "
+                  "in '--default-language %s'. See 'mkvmerge --list-languages'"
+                  "for a list of all languages and their respective ISO639-2 "
+                  "codes.\n"),
+                next_arg.c_str(), next_arg.c_str());
+
+      default_language = iso639_languages[i].iso639_2_code;
       sit++;
 
     } else if (this_arg == "--sub-charset") {
@@ -1845,16 +2018,10 @@ parse_args(vector<string> args) {
       if (no_next_arg)
         mxerror(_("'%s' lacks the track ID.\n"), this_arg.c_str());
 
-      if (!parse_int(next_arg, id) || (id < 0))
-        mxerror(_("Invalid track ID specified in '%s %s'.\n"),
-                this_arg.c_str(), next_arg.c_str());
-
-      ti->aac_is_sbr.push_back(id);
+      parse_aac_is_sbr(next_arg, *ti);
       sit++;
 
     } else if (this_arg == "--compression") {
-      compression_method_t compression;
-
       if (no_next_arg)
         mxerror(_("'--compression' lacks its argument.\n"));
 
@@ -1869,24 +2036,19 @@ parse_args(vector<string> args) {
       sit++;
 
     } else if (this_arg == "--track-name") {
-      language_t lang;
-
       if (no_next_arg)
         mxerror(_("'--track-name' lacks its argument.\n"));
 
-      parse_language(next_arg, lang, "track-name", "track name", false, true);
-      ti->track_names.push_back(track_name_t(lang.language, lang.id));
+      parse_language(next_arg, ti->track_names, "track-name", "track name",
+                     false, true);
       sit++;
 
     } else if (this_arg == "--timecodes") {
-      language_t lang;
-      string s;
-
       if (no_next_arg)
         mxerror(_("'--timecodes' lacks its argument.\n"));
 
-      parse_language(next_arg, lang, "timecodes", "timecodes", false);
-      ti->all_ext_timecodes.push_back(ext_timecodes_t(lang.language, lang.id));
+      parse_language(next_arg, ti->all_ext_timecodes, "timecodes",
+                     "timecodes", false);
       sit++;
 
     } else if (this_arg == "--track-order") {
@@ -1912,6 +2074,13 @@ parse_args(vector<string> args) {
       parse_default_duration(next_arg, *ti);
       sit++;
 
+    } else if (this_arg == "--nalu-size-length") {
+      if (no_next_arg)
+        mxerror(_("'--nalu-size-length' lacks its argument.\n"));
+
+      parse_nalu_size_length(next_arg, *ti);
+      sit++;
+
     } else if (this_arg == "--print-malloc-report")
       print_malloc_report = true;
 
@@ -1977,6 +2146,9 @@ parse_args(vector<string> args) {
     mxwarn(_("'--link' is only useful in combination with '--split'.\n"));
 
   delete ti;
+
+  if (!inputs_found && (0 == files.size()))
+    mxerror("No input files were given. No output will be created.\n");
 }
 
 /** \brief Initialize global variables
@@ -2017,7 +2189,7 @@ main(int argc,
   finish_file(true);
 
   end = time(NULL);
-  mxinfo(_("Muxing took %ld second%s.\n"), end - start,
+  mxinfo(_("Muxing took %ld second%s.\n"), (long)(end - start),
          (end - start) == 1 ? "" : "s");
 
   cleanup();
