@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.cpp 3031 2005-08-21 13:04:48Z mosu $
+   $Id: mmg.cpp 3463 2007-01-12 16:02:16Z mosu $
 
    main stuff
 
@@ -38,7 +38,9 @@
 #include "chapters.h"
 #include "common.h"
 #include "commonebml.h"
+#include "extern_data.h"
 #include "jobs.h"
+#include "mkvmerge.h"
 #include "matroskalogo.xpm"
 #include "mmg.h"
 #include "mmg_dialog.h"
@@ -131,9 +133,11 @@ wxString cli_options[][2] = {
   { wxT("--engage force_passthrough_packetizer"),
     wxT("Forces the Matroska reader to use the generic passthrough "
         "packetizer even for known and supported track types.") },
-  { wxT("--engage allow_avc_in_vfw_mode"),
-    wxT("Allows storing AVC/h.264 video in Video-for-Windows compatibility "
-        "mode, e.g. when it is read from an AVI") },
+  { wxT("--engage use_simpleblock"),
+    wxT("Enable use of SimpleBlock instead of BlockGroup when possible.") },
+  { wxT("--engage old_aac_codecid"),
+    wxT("Use the old AAC codec IDs (e.g. 'A_AAC/MPEG4/SBR') instead of the "
+        "new one ('A_AAC').") },
   { wxT("--engage cow"),
     wxT("No help available.") },
   { wxT(""), wxT("") }
@@ -544,8 +548,10 @@ wxString
 create_track_order(bool all) {
   int i;
   wxString s, format;
+  string temp;
 
-  format = wxT("%d:" LLD);
+  fix_format("%d:" LLD, temp);
+  format = wxU(temp.c_str());
   for (i = 0; i < tracks.size(); i++) {
     if (!all && (!tracks[i]->enabled || tracks[i]->appending))
       continue;
@@ -561,8 +567,10 @@ wxString
 create_append_mapping() {
   int i;
   wxString s, format;
+  string temp;
 
-  format = wxT("%d:" LLD ":%d:" LLD);
+  fix_format("%d:" LLD ":%d:" LLD, temp);
+  format = wxU(temp.c_str());
   for (i = 1; i < tracks.size(); i++) {
     if (!tracks[i]->enabled || !tracks[i]->appending)
       continue;
@@ -585,6 +593,20 @@ default_track_checked(char type) {
   return -1;
 }
 
+void
+set_combobox_selection(wxComboBox *cb,
+                       const wxString wanted) {
+  int i, count;
+
+  cb->SetValue(wanted);
+  count = cb->GetCount();
+  for (i = 0; count > i; ++i)
+    if (cb->GetString(i) == wanted) {
+      cb->SetSelection(i);
+      break;
+    }
+}
+
 void
 wxdie(const wxString &errmsg) {
   wxMessageBox(errmsg, wxT("A serious error has occured"),
@@ -641,7 +663,7 @@ mmg_dialog::mmg_dialog():
                       wxT("Adds the current settings as a new job entry to "
                           "the job queue"));
   muxing_menu->Append(ID_M_MUXING_MANAGE_JOBS,
-                      wxT("&Manage jobs"),
+                      wxT("&Manage jobs\tCtrl-j"),
                       wxT("Brings up the job queue editor"));
   muxing_menu->AppendSeparator();
   muxing_menu->Append(ID_M_MUXING_ADD_CLI_OPTIONS,
@@ -877,9 +899,11 @@ mmg_dialog::on_file_new(wxCommandEvent &evt) {
   tc_output->SetValue(wxT(""));
 
   input_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  input_page->on_file_new(evt);
   attachments_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
   global_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
   settings_page->load(cfg, MMG_CONFIG_FILE_VERSION_MAX);
+  notebook->SetSelection(0);
 
   delete cfg;
   wxRemoveFile(tmp_name);
@@ -980,17 +1004,14 @@ mmg_dialog::save(wxString file_name,
 void
 mmg_dialog::set_last_settings_in_menu(wxString name) {
   uint32_t i;
-  vector<wxString>::iterator eit;
   wxConfigBase *cfg;
   wxString s;
 
   i = 0;
   while (i < last_settings.size()) {
-    if (last_settings[i] == name) {
-      eit = last_settings.begin();
-      eit += i;
-      last_settings.erase(eit);
-    } else
+    if (last_settings[i] == name)
+      last_settings.erase(last_settings.begin() + i);
+    else
       i++;
   }
   last_settings.insert(last_settings.begin(), name);
@@ -1011,17 +1032,14 @@ mmg_dialog::set_last_settings_in_menu(wxString name) {
 void
 mmg_dialog::set_last_chapters_in_menu(wxString name) {
   uint32_t i;
-  vector<wxString>::iterator eit;
   wxConfigBase *cfg;
   wxString s;
 
   i = 0;
   while (i < last_chapters.size()) {
-    if (last_chapters[i] == name) {
-      eit = last_chapters.begin();
-      eit += i;
-      last_chapters.erase(eit);
-    } else
+    if (last_chapters[i] == name)
+      last_chapters.erase(last_chapters.begin() + i);
+    else
       i++;
   }
   last_chapters.insert(last_chapters.begin(), name);
@@ -1048,13 +1066,6 @@ mmg_dialog::check_before_overwriting() {
 
   dir = file_name.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
 
-  if (!wxDir::Exists(dir)) {
-    wxMessageBox(wxT("The output directory '") + dir +
-                 wxT("' does not exist."),
-                 wxT("Directory does not exist"), wxOK | wxICON_ERROR);
-    return false;
-  }
-
   if (!global_page->cb_split->GetValue()) {
     if (wxFile::Exists(tc_output->GetValue()) &&
         (wxMessageBox(wxT("The output file '") + tc_output->GetValue() +
@@ -1155,16 +1166,54 @@ mmg_dialog::on_help(wxCommandEvent &evt) {
   if (help == NULL) {
     wxDirDialog dlg(this, wxT("Chosoe the location of the mkvmerge GUI "
                               "help files"));
+    vector<wxString> potential_help_paths;
+    vector<wxString>::const_iterator php;
     wxString help_path;
     wxConfigBase *cfg;
     bool first;
 
-    help_path = wxGetCwd();
     cfg = wxConfigBase::Get();
     cfg->SetPath(wxT("/GUI"));
-    if (!cfg->Read(wxT("help_path"), &help_path))
-      if (cfg->Read(wxT("installation_path"), &help_path))
-        help_path += wxT("/doc");
+
+    if (cfg->Read(wxT("installation_path"), &help_path)) {
+      help_path += wxT("/doc");
+      potential_help_paths.push_back(help_path);
+    }
+#if !defined(SYS_WINDOWS)
+    // Debian, probably others
+    potential_help_paths.push_back(wxT("/usr/share/doc/mkvtoolnix"));
+    potential_help_paths.push_back(wxT("/usr/share/doc/mkvtoolnix/doc"));
+    potential_help_paths.push_back(wxT("/usr/share/doc/mkvtoolnix-gui"));
+    // SuSE
+    potential_help_paths.push_back(wxT("/usr/share/doc/packages/mkvtoolnix"));
+    // Fedora Core
+    potential_help_paths.push_back(wxT("/usr/share/doc/mkvtoolnix-" VERSION));
+    potential_help_paths.push_back(wxT("/usr/share/doc/mkvtoolnix-gui-"
+                                       VERSION));
+    // (Almost the) same for /usr/local
+    potential_help_paths.push_back(wxT("/usr/local/share/doc/mkvtoolnix"));
+    potential_help_paths.push_back(wxT("/usr/local/share/doc/packages/"
+                                       "mkvtoolnix"));
+    potential_help_paths.push_back(wxT("/usr/local/share/doc/mkvtoolnix-"
+                                       VERSION));
+    potential_help_paths.push_back(wxT("/usr/local/share/doc/mkvtoolnix-gui-"
+                                       VERSION));
+    // New location
+    potential_help_paths.push_back(wxT(MTX_PKG_DATA_DIR));
+    potential_help_paths.push_back(wxT(MTX_PKG_DATA_DIR "-" VERSION));
+#endif
+    if (cfg->Read(wxT("help_path"), &help_path))
+      potential_help_paths.push_back(help_path);
+    potential_help_paths.push_back(wxGetCwd() + wxT("/doc"));
+    potential_help_paths.push_back(wxGetCwd());
+
+    help_path = wxT("");
+    mxforeach(php, potential_help_paths)
+      if (wxFileExists(*php + wxT("/mkvmerge-gui.hhp"))) {
+        help_path = *php;
+        break;
+      }
+
     first = true;
     while (!wxFileExists(help_path + wxT("/mkvmerge-gui.hhp"))) {
       if (first) {
@@ -1187,8 +1236,8 @@ mmg_dialog::on_help(wxCommandEvent &evt) {
       if (dlg.ShowModal() == wxID_CANCEL)
         return;
       help_path = dlg.GetPath();
+      cfg->Write(wxT("help_path"), help_path);
     }
-    cfg->Write(wxT("help_path"), help_path);
     help = new wxHtmlHelpController;
     help->AddBook(wxFileName(help_path + wxT("/mkvmerge-gui.hhp")), false);
   }
@@ -1339,9 +1388,10 @@ mmg_dialog::update_command_line() {
           aids += wxT(",");
         aids += sid;
 
-        if (!t->appending && t->aac_is_sbr) {
+        if (!t->appending && (t->aac_is_sbr || t->aac_is_sbr_detected)) {
           clargs.Add(wxT("--aac-is-sbr"));
-          clargs.Add(sid);
+          clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
+                                      t->aac_is_sbr ? 1 : 0));
         }
 
       } else if (t->type == wxT('v')) {
@@ -1424,6 +1474,26 @@ mmg_dialog::update_command_line() {
         clargs.Add(sid + wxT(":") + t->fourcc);
       }
 
+      if (t->fps.Length() > 0) {
+        clargs.Add(wxT("--default-duration"));
+        clargs.Add(sid + wxT(":") + t->fps + wxT("fps"));
+      }
+
+      if ((FILE_TYPE_AVC_ES == f->container) &&
+          ('v' == t->type) &&
+          (t->ctype.Find(wxT("MPEG-4 part 10 ES")) >= 0) &&
+          (2 != t->nalu_size_length)) {
+        clargs.Add(wxT("--nalu-size-length"));
+        clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
+                                    t->nalu_size_length));
+      }
+
+      if (!t->appending && (0 != t->stereo_mode)) {
+        clargs.Add(wxT("--stereo-mode"));
+        clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
+                                    t->stereo_mode - 1));
+      }
+
       if (!t->appending && (t->compression.Length() > 0)) {
         clargs.Add(wxT("--compression"));
         clargs.Add(sid + wxT(":") + t->compression);
@@ -1433,6 +1503,18 @@ mmg_dialog::update_command_line() {
         clargs.Add(wxT("--timecodes"));
         clargs.Add(sid + wxT(":") + t->timecodes);
       }
+
+      if (t->user_defined.Length() > 0) {
+        vector<wxString> opts;
+        int i;
+
+        opts = split(t->user_defined, wxString(wxT(" ")));
+        for (i = 0; i < opts.size(); i++) {
+          wxString opt = strip(opts[i]);
+          opt.Replace(wxT("<TID>"), sid, true);
+          clargs.Add(opt);
+        }
+      }
     }
 
     if (aids.length() > 0) {
@@ -1491,6 +1573,8 @@ mmg_dialog::update_command_line() {
       clargs.Add(wxT("--attachment-description"));
       clargs.Add(no_cr(a->description));
     }
+    clargs.Add(wxT("--attachment-name"));
+    clargs.Add(a->stored_name);
     if (a->style == 0)
       clargs.Add(wxT("--attach-file"));
     else
@@ -1569,6 +1653,11 @@ mmg_dialog::update_command_line() {
       clargs.Add(strip(opts[i]));
   }
 
+  if (settings_page->cb_always_use_simpleblock->IsChecked()) {
+    clargs.Add(wxT("--engage"));
+    clargs.Add(wxT("use_simpleblock"));
+  }
+
   for (i = args_start; i < clargs.Count(); i++) {
     if (clargs[i].Find(wxT(" ")) >= 0)
       cmdline += wxT(" \"") + shell_escape(clargs[i]) + wxT("\"");
@@ -1700,17 +1789,31 @@ mmg_dialog::set_title_maybe(const wxString &new_title) {
 
 void
 mmg_dialog::set_output_maybe(const wxString &new_output) {
-  wxString output;
-
   if (settings_page->cb_autoset_output_filename->IsChecked() &&
       (new_output.length() > 0) &&
       (tc_output->GetValue().length() == 0)) {
-    output = new_output.BeforeLast('.');
-    output += wxT(".mkv");
+    wxString output(new_output.BeforeLast('.'));
+    bool has_video = false, has_audio = false;
+    vector<mmg_track_t *>::iterator t;
+
+    mxforeach(t, tracks) {
+      if ('v' == (*t)->type) {
+        has_video = true;
+        break;
+      } else if ('a' == (*t)->type)
+        has_audio = true;
+    }
+    output += has_video ? wxT(".mkv") : has_audio ? wxT(".mka") : wxT(".mks");
     tc_output->SetValue(output);
   }
 }
 
+void
+mmg_dialog::remove_output_filename() {
+  if (settings_page->cb_autoset_output_filename->IsChecked())
+    tc_output->SetValue(wxT(""));
+}
+
 void
 mmg_dialog::on_add_to_jobqueue(wxCommandEvent &evt) {
   wxString description, line;
@@ -1993,6 +2096,7 @@ mmg_app::OnInit() {
   wxConfigBase *cfg;
   uint32_t i;
   wxString k, v;
+  int index;
 
   init_stdio();
   mm_file_io_c::setup();
@@ -2014,11 +2118,14 @@ mmg_app::OnInit() {
   cfg->SetPath(wxT("/chapter_editor"));
   cfg->Read(wxT("default_language"), &k, wxT("und"));
   default_chapter_language = to_utf8(k).c_str();
-  if (!is_valid_iso639_2_code(default_chapter_language.c_str()))
+  index = map_to_iso639_2_code(default_chapter_language.c_str());
+  if (-1 == index)
     default_chapter_language = "und";
+  else
+    default_chapter_language = iso639_languages[index].iso639_2_code;
   if (cfg->Read(wxT("default_country"), &k) && (0 < k.length()))
     default_chapter_country = to_utf8(k).c_str();
-  if (!is_valid_iso639_1_code(default_chapter_country.c_str()))
+  if (!is_valid_cctld(default_chapter_country.c_str()))
     default_chapter_country = "";
 
   app = this;
