@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_flac.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: p_flac.cpp 3194 2006-02-27 17:06:55Z mosu $
 
    FLAC packetizer
 
@@ -44,16 +44,17 @@ flac_packetizer_c::flac_packetizer_c(generic_reader_c *_reader,
 
   if ((_l_header < 4) || (_header[0] != 'f') || (_header [1] != 'L') ||
       (_header[2] != 'a') || (_header[3] != 'C')) {
-    header.set(safemalloc(_l_header + 4));
-    memcpy(header.get(), "fLaC", 4);
-    memcpy(header.get() + 4, _header, _l_header);
-    l_header = _l_header + 4;
-  } else {
-    header.set(safememdup(_header, _l_header));
-    l_header = _l_header;
-  }
+    header = memory_cptr(new memory_c((unsigned char *)
+                                      safemalloc(_l_header + 4),
+                                      _l_header + 4, true));
+    memcpy(header->get(), "fLaC", 4);
+    memcpy(header->get() + 4, _header, _l_header);
+  } else
+    header = memory_cptr(new memory_c((unsigned char *)
+                                      safememdup(_header, _l_header),
+                                      _l_header, true));
 
-  result = flac_decode_headers(header, l_header, 1,
+  result = flac_decode_headers(header->get(), header->get_size(), 1,
                                FLAC_HEADER_STREAM_INFO, &stream_info);
   if (!(result & FLAC_HEADER_STREAM_INFO))
     mxerror(_(FMT_TID "The FLAC headers could not be parsed: the stream info "
@@ -78,7 +79,7 @@ flac_packetizer_c::~flac_packetizer_c() {
 void
 flac_packetizer_c::set_headers() {
   set_codec_id(MKV_A_FLAC);
-  set_codec_private(header, l_header);
+  set_codec_private(header->get(), header->get_size());
   set_audio_sampling_freq((float)stream_info.sample_rate);
   set_audio_channels(stream_info.channels);
   set_audio_bit_depth(stream_info.bits_per_sample);
@@ -90,10 +91,11 @@ int
 flac_packetizer_c::process(packet_cptr packet) {
   debug_enter("flac_packetizer_c::process");
 
-  packet->duration = flac_get_num_samples(packet->memory->data,
-                                          packet->memory->size, stream_info);
+  packet->duration =
+    flac_get_num_samples(packet->data->get(), packet->data->get_size(),
+                         stream_info);
   if (packet->duration == -1) {
-    mxwarn(_(FMT_TID "Packet number %lld contained an invalid FLAC header "
+    mxwarn(_(FMT_TID "Packet number " LLD " contained an invalid FLAC header "
              "and is being skipped.\n"), ti.fname.c_str(), (int64_t)ti.id,
            num_packets + 1);
     debug_leave("flac_packetizer_c::process");
@@ -128,13 +130,13 @@ flac_packetizer_c::can_connect_to(generic_packetizer_c *src,
   connect_check_a_bitdepth(stream_info.bits_per_sample,
                            fsrc->stream_info.bits_per_sample);
 
-  if ((l_header != fsrc->l_header) ||
+  if ((header->get_size() != fsrc->header->get_size()) ||
       (NULL == header.get()) || (NULL == fsrc->header.get()) ||
-      memcmp(header.get(), fsrc->header.get(), l_header)) {
+      memcmp(header->get(), fsrc->header->get(), header->get_size())) {
     error_message = mxsprintf("The FLAC header data is different for the "
                               "two tracks (lengths: %d and %d)",
-                              l_header, fsrc->l_header);
-    return CAN_CONNECT_NO_PARAMETERS;
+                              header->get_size(), fsrc->header->get_size());
+    return CAN_CONNECT_MAYBE_CODECPRIVATE;
   }
   return CAN_CONNECT_YES;
 }
