@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_video.cpp 3194 2006-02-27 17:06:55Z mosu $
+   $Id: p_video.cpp 3458 2007-01-11 19:01:55Z mosu $
 
    video output module
 
@@ -66,48 +66,6 @@ video_packetizer_c::check_fourcc() {
            ti.fourcc.c_str(), 4);
     set_codec_private(ti.private_data, ti.private_size);
   }
-
-  if (!hack_engaged(ENGAGE_ALLOW_AVC_IN_VFW_MODE) &&
-      (hcodec_id == MKV_V_MSCOMP) &&
-      (ti.private_data != NULL) &&
-      (ti.private_size >= sizeof(alBITMAPINFOHEADER))) {
-    const char *avc_fourccs[] = {
-      "h264", "x264", "avc1", NULL
-    };
-    const char *fourcc;
-    int i;
-
-    fourcc = (const char *)
-      &((alBITMAPINFOHEADER *)ti.private_data)->bi_compression;
-    for (i = 0; NULL != avc_fourccs[i]; ++i)
-      if (!strncasecmp(fourcc, avc_fourccs[i], 4))
-        mxerror(FMT_TID "You are trying to put AVC/h.264 video from an AVI "
-                "or a similar VfW (Video for Windows) compatible source into "
-                "a Matroska file in the so-called 'VfW compatibility mode'. "
-                "Please note that this is not the official way to store "
-                "AVC/h.264 video in Matroska. Therefore proper playback of "
-                "such files cannot be guaranteed, and we strongly urge you "
-                "to use the native Matroska-mode.\n"
-                "At the moment mkvmerge does not support converting from "
-                "VfW-mode AVC/h.264 tracks to native Matroska-mode AVC/h.264 "
-                "tracks. You can, however, first import the video track "
-                "into a MP4 file with e.g. 'MP4Box' (use Google). Then "
-                "you can use mkvmerge and put the video into a Matroska "
-                "file.\n"
-                "If you're trying to get AVC/h.264 from an AVI into Matroska "
-                "then the commands you need to use are for example:\n"
-                " avi2raw avc.avi raw.264\n"
-                " mp4box -fps 23.976 -add raw.264 avc.mp4\n\n"
-                "Keep in mind to replace '-fps 23.976' with the actual "
-                "frame rate of your source file.\n"
-                "If you really know what you are doing then you can force "
-                "mkvmerge to put this AVC/h.264 track into a Matroska file "
-                "even in VfW mode if you add '--engage allow_avc_in_vfw_mode' "
-                "to the command line. You can do that in mmg with the "
-                "'Add command line options' menu entry in the 'Muxing' "
-                "menu.\n",
-                ti.fname.c_str(), (int64_t)ti.id);
-  }
 }
 
 void
@@ -273,8 +231,8 @@ mpeg1_2_video_packetizer_c::process(packet_cptr packet) {
       if (frame == NULL)
         break;
 
-      if (hcodec_private == NULL)
-        create_private_data();
+//       if (hcodec_private == NULL)
+//         create_private_data();
 
       packet_t *new_packet =
         new packet_t(new memory_c(frame->data, frame->size, true),
@@ -306,9 +264,9 @@ mpeg1_2_video_packetizer_c::extract_fps(const unsigned char *buffer,
                                         int size) {
   int idx;
 
-  idx = mpeg1_2_extract_fps_idx(buffer, size);
+  idx = mpeg1_2::extract_fps_idx(buffer, size);
   if (idx >= 0) {
-    fps = mpeg1_2_get_fps(idx);
+    fps = mpeg1_2::get_fps(idx);
     if (fps > 0) {
       set_track_default_duration((int64_t)(1000000000.0 / fps));
       rerender_track_headers();
@@ -325,7 +283,7 @@ mpeg1_2_video_packetizer_c::extract_aspect_ratio(const unsigned char *buffer,
   if (ti.aspect_ratio_given || ti.display_dimensions_given)
     return;
 
-  if (mpeg1_2_extract_ar(buffer, size, ar)) {
+  if (mpeg1_2::extract_ar(buffer, size, ar)) {
     ti.display_dimensions_given = true;
     if ((ar <= 0) || (ar == 1))
       set_video_display_width(width);
@@ -406,8 +364,8 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
   if (NULL == ti.private_data) {
     memory_c *config_data;
 
-    config_data = mpeg4_p2_parse_config_data(packet->data->get(),
-                                             packet->data->get_size());
+    config_data = mpeg4::p2::parse_config_data(packet->data->get(),
+                                               packet->data->get_size());
     if (NULL != config_data) {
       ti.private_data = (unsigned char *)safememdup(config_data->get(),
                                                     config_data->get_size());
@@ -423,8 +381,8 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
               "native mode.\n");
   }
 
-  mpeg4_p2_find_frame_types(packet->data->get(), packet->data->get_size(),
-                            frames);
+  mpeg4::p2::find_frame_types(packet->data->get(), packet->data->get_size(),
+                              frames);
 
   // Add a timecode and a duration if they've been given.
   if (-1 != packet->timecode)
@@ -436,7 +394,7 @@ mpeg4_p2_video_packetizer_c::process_non_native(packet_cptr packet) {
   if (-1 != packet->duration)
     available_durations.push_back(packet->duration);
 
-  foreach(frame, frames) {
+  mxforeach(frame, frames) {
     // Maybe we can flush queued frames now. But only if we don't have
     // a B frame.
     if (FRAME_TYPE_B != frame->type)
@@ -672,7 +630,7 @@ mpeg4_p2_video_packetizer_c::extract_aspect_ratio(const unsigned char *buffer,
     return;
   }
 
-  if (mpeg4_p2_extract_par(buffer, size, num, den)) {
+  if (mpeg4::p2::extract_par(buffer, size, num, den)) {
     aspect_ratio_extracted = true;
     ti.aspect_ratio_given = true;
     ti.aspect_ratio = (float)hvideo_pixel_width /
@@ -692,7 +650,7 @@ mpeg4_p2_video_packetizer_c::extract_size(const unsigned char *buffer,
                                           int size) {
   uint32_t width, height;
 
-  if (mpeg4_p2_extract_size(buffer, size, width, height)) {
+  if (mpeg4::p2::extract_size(buffer, size, width, height)) {
     size_extracted = true;
     if (!reader->appending &&
         ((width != hvideo_pixel_width) || (height != hvideo_pixel_height))) {
@@ -728,6 +686,8 @@ mpeg4_p10_video_packetizer_c(generic_reader_c *_reader,
                              track_info_c &_ti):
   video_packetizer_c(_reader, MKV_V_MPEG4_AVC, _fps, _width, _height, _ti) {
 
+  relaxed_timecode_checking = true;
+
   if ((ti.private_data != NULL) && (ti.private_size > 0))
     extract_aspect_ratio();
 }
@@ -738,7 +698,7 @@ mpeg4_p10_video_packetizer_c::extract_aspect_ratio() {
   unsigned char *priv;
 
   priv = ti.private_data;
-  if (mpeg4_p10_extract_par(ti.private_data, ti.private_size, num, den) &&
+  if (mpeg4::p10::extract_par(ti.private_data, ti.private_size, num, den) &&
       (0 != num) && (0 != den)) {
     if (!ti.aspect_ratio_given && !ti.display_dimensions_given) {
       double par = (double)num / (double)den;
